/* addfile.h -- Copyright 1989 Liam R. Quin.  All Rights Reserved.
 * This code is NOT in the public domain.
 * See the file COPYRIGHT for full details.
 */

/* $Id: addfile.h,v 1.4 92/07/21 19:01:25 lee Exp $
 *
 * This file defines the parameter to DumpCache.
 *
 */

#define MAXPROBES	35
    /* If we don't find an empty cache slot after this many tries,
     * we decide the hash table is full and clear it out, then try again.
     */

#define DUMP_CACHE	00	/* normal usage */
#define DUMP_SYNC	01	/* write all cached entries to disk */
#define DUMP_NOFREE	02	/* don't bother calling free() */
#define DUMP_FAST	04	/* only do a fast dump */

/* when the word-cache fills up, we first try and dump all of
 * the words that occur DUMP_FAST_THRESH or fewer times.  This generally
 * clears up a lot of slots in the hash table.  If that hasn't cleared
 * enough memory, we then dump the single most frequent word, before
 * increasing the dump threshhold and trying again.
 * Now that updating the index is possible, I am not sure that this is
 * the best way to continue, but DUMP_FAST_THRESH is certainly an effective
 * way to clear slots in the cache.
 * MUST_DUMP is used to start the dump process.
 */

#ifndef DUMP_FAST_THRESH
# define DUMP_FAST_THRESH	5
#endif

/* MUST_DUMP evaluates to non-zero when lqaddfile must dump its cache;;
 * MUST_DUMP_MORE is used to see if it's OK to stop dumping.
 */
#ifndef MUST_DUMP
# define MUST_DUMP(InCacheCount,MaxInCache,SlotsUsed,TotalSlots) \
	( (InCacheCount + 1 >= MaxInCache) || \
	  (SlotsUsed * 10 > TotalSlots * 9)) /* allow 90% full */
#endif

/* MUST_DUMP_MORE returns 0 when the cache is suffiently dumped to
 * enable more indexing to proceed.
 */
#ifndef MUST_DUMP_MORE
# define MUST_DUMP_MORE(InCacheCount,MaxInCache,SlotsUsed,TotalSlots) \
	( (InCacheCount * 100 >= MaxInCache * 80) /* allow 80% full */ || \
	  (SlotsUsed * 10 > TotalSlots * 3)) /* allow 30% full */
#endif

/* FILE_DUMP returns 1 if we should dump the cache between files:
 */
#ifndef MUST_FILE_DUMP
# define MUST_FILE_DUMP(InCacheCount,MaxInCache,SlotsUsed,TotalSlots) \
	( (InCacheCount * 100 >= MaxInCache * 99)  || \
	  (SlotsUsed * 100 > TotalSlots * 75)) /* allow 75% full */
#endif

/* FILE_DUMP_MORE returns 1 if we haven't finished dumping between files:
 */
#ifndef MUST_FILE_DUMP_MORE
# define MUST_FILE_DUMP_MORE(InCacheCount,MaxInCache,SlotsUsed,TotalSlots) \
	( (InCacheCount * 100 >= MaxInCache * 90) /* allow 90% full */ || \
	  (SlotsUsed * 100 > TotalSlots * 60)) /* allow 60% full */
#endif

